@@ -1,4 +1,5 @@
 from datetime import datetime
+import binascii
 
 
 MAGNITUDE_W = 1
@@ -11,6 +12,22 @@ MAGNITUDE_KW = 1000
 Magnitude value (1000) for measures represented in kW.
 """
 
+SAGE_BAD_TIMESTAMP = [
+    'FFFFFFFFFFFFFFW',
+    'FFFFFFFF000000S',
+]
+
+S23_BAD_TIMESTAMP = [
+    '00000000000000W',
+    '00000101000000W',
+    '000000000000000',
+    'FFFFFFFFFFFFFF9',
+    'FFFFFFFFFFFFFF0',
+    'FFFFFFFFFFFFFFF'
+]
+
+BAD_TIMESTAMP = SAGE_BAD_TIMESTAMP + S23_BAD_TIMESTAMP
+
 
 class ValueWithTime(object):
     """
@@ -39,9 +56,12 @@ class ValueWithTime(object):
 
         # Fix for SAGECOM which puts this timestamp when the period doesn't
         # affect the contracted tariff
-        if date_value.upper() == 'FFFFFFFFFFFFFFW':
+        if date_value.upper() in BAD_TIMESTAMP or not date_value:
             date_value = '19000101000000W'
 
+        if date_value.startswith('ffff'):
+            date_value = date_value.replace(date_value[:4], '9999')
+
         try:
             time = datetime.strptime(date_value[:-1],
                                      '%Y%m%d%H%M%S')
@@ -122,6 +142,31 @@ class MeasureActiveReactive(Measure):
         }
 
 
+class MeasureActiveReactiveFloat(Measure):
+    """
+    Base class for a set of measures with active and reactive measures.
+    """
+
+    def active_reactive(self, measure, measure_type):
+        """
+        Get the active and reactive measures.
+
+        :param measure: an lxml.objectify.StringElement representing a set of \
+            measures
+        :param measure_type: the type of measure, added at the end of the \
+            name of each measure ('a' or 'i')
+        :return: a dict with the active and reactive measures
+        """
+        return {
+            'ai': float(measure.get('AI{}'.format(measure_type))),
+            'ae': float(measure.get('AE{}'.format(measure_type))),
+            'r1': float(measure.get('R1{}'.format(measure_type))),
+            'r2': float(measure.get('R2{}'.format(measure_type))),
+            'r3': float(measure.get('R3{}'.format(measure_type))),
+            'r4': float(measure.get('R4{}'.format(measure_type))),
+        }
+
+
 class Parameter(ValueWithTime):
     """
     Base class for a set of parameters.
